@@ -17,7 +17,6 @@
 # ------------------------------------------------------------------------------
 from concurrent import futures
 import os
-from pathlib import Path
 import grpc
 
 from colearn_grpc.mli_factory_interface import MliFactory
@@ -27,9 +26,6 @@ import colearn_grpc.proto.generated.interface_pb2_grpc as ipb2_grpc
 
 from colearn_grpc.logging import get_logger
 
-REPO_ROOT = Path(__file__).absolute().parent.parent
-server_key = REPO_ROOT / "server.key"
-server_crt = REPO_ROOT / "server.crt"
 
 _logger = get_logger(__name__)
 
@@ -41,17 +37,24 @@ class GRPCServer:
         object, and creates the GRPC listener server, which can be started using the run method.
     """
 
-    def __init__(self, mli_factory: MliFactory, port=None, max_workers=5):
+    def __init__(self, mli_factory: MliFactory, port=None, max_workers=5,
+                 enable_encryption=False, server_key="", server_crt=""):
         """
             @param mli_factory is a factory object that produces MachineLearningInterface objects
             @param port is the port where the server will listen
             @param max_workers is how many worker threads will be available in the thread pool
+            @param enable_encryption if True they the server will try to enable encryption
+            @param server_key path to server key for encryption (if enabled)
+            @param server_crt path to server certificate for encryption (if enabled)
         """
         self.port = port
         self.server = None
         self.service = GRPCLearnerServer(mli_factory)
         self.thread_pool = None
         self.max_workers = max_workers
+        self.enable_encryption = enable_encryption
+        self.server_key = server_key
+        self.server_crt = server_crt
 
     def run(self):
         if self.server:
@@ -61,17 +64,19 @@ class GRPCServer:
 
         _logger.info(f"Starting GRPC server on {address}...")
 
-        # There needs to be a certificate and private key available to enable encryption -
-        # if these are not available, fall back to no encryption.
-        encrypted_connection = True
+        encrypted_connection = False
+        if self.enable_encryption:
+            # There needs to be a certificate and private key available to enable encryption -
+            # if these are not available, fall back to no encryption.
+            encrypted_connection = True
 
-        if not os.path.isfile(server_crt):
-            _logger.error(f"Failed to find file {server_crt} needed for encrypted grpc connection - not enabling")
-            encrypted_connection = False
+            if not os.path.isfile(self.server_crt):
+                _logger.error(f"Failed to find file {self.server_crt} needed for encrypted grpc connection - not enabling")
+                encrypted_connection = False
 
-        if not os.path.isfile(server_key):
-            _logger.error(f"Failed to find file {server_key} needed for encrypted grpc connection - not enabling")
-            encrypted_connection = False
+            if not os.path.isfile(self.server_key):
+                _logger.error(f"Failed to find file {self.server_key} needed for encrypted grpc connection - not enabling")
+                encrypted_connection = False
 
         self.thread_pool = futures.ThreadPoolExecutor(
             max_workers=self.max_workers, thread_name_prefix="GRPCLearnerServer-poolworker-")
@@ -81,9 +86,9 @@ class GRPCServer:
 
         if encrypted_connection:
             # read in key and certificate
-            with open(server_key, 'rb') as f:
+            with open(self.server_key, 'rb') as f:
                 private_key = f.read()
-            with open(server_crt, 'rb') as f:
+            with open(self.server_crt, 'rb') as f:
                 certificate_chain = f.read()
 
             # create server credentials
